from django.shortcuts import render, redirect, get_object_or_404
from django.contrib.auth.decorators import login_required
from django.contrib.auth.mixins import LoginRequiredMixin
from django.http import HttpResponse, HttpResponseNotFound
from django.views.generic import View
from django.forms import formset_factory
from django.contrib import messages
from .forms import MediaForm, BlogForm, BlogDeleteForm
from .models import Photo, Blog


def home(request):
    context = dict()
    return render(request, 'blog/home.html', context)


def not_found(request, exception):
    print("enter not found")
    return render(request,
                  '404.html',
                  context={"exception": exception},
                  status=404)


class MediaCreateView(LoginRequiredMixin, View):
    context = dict()
    template_name = 'blog/media_create.html'
    form_class = MediaForm

    def get(self, request):
        self.context['form'] = self.form_class()
        return render(request, self.template_name, self.context)

    def post(self, request):
        self.context['form'] = form = self.form_class(request.POST,
                                                       request.FILES)
        if form.is_valid():
            photo = form.save(commit=False)
            photo.uploader = request.user
            photo.save()
            messages.add_message(request,
                                 messages.INFO,
                                 'photo ajoutée avec succès')
            return redirect('home')
        return render(request, self.template_name, self.context)

class MediaMultipleCreateView(LoginRequiredMixin, View):
    context = dict()
    template_name = 'blog/media_multiple_create.html'
    MediaFormset = formset_factory(MediaForm, extra=5)

    def get(self, request):
        self.context['media_formset'] = self.MediaFormset()
        return render(request, self.template_name, self.context)

    def post(self, request):
        self.context['media_formset'] = media_formset = self.MediaFormset(request.POST,
                                                                           request.FILES)
        if (media_formset.is_valid()):
            for form in media_formset:
                if form.cleaned_data:
                    photo = form.save(commit=False)
                    photo.uploader = request.user
                    photo.save()
            return redirect('media-list')

        render(request, self.template_name, self.context)


class PostCreateView(LoginRequiredMixin, View):
    context = dict()
    template_name = 'blog/post_create.html'
    blog_form_class = BlogForm
    media_form_class = MediaForm

    def get(self, request):
        self.context['blog_form'] = self.blog_form_class()
        self.context['media_form'] = self.media_form_class()
        return render(request, self.template_name, self.context)

    def post(self, request):
        self.context['blog_form'] = blog_form = self.blog_form_class(request.POST)
        self.context['media_form'] = media_form = self.media_form_class(request.POST,
                                                                        request.FILES)
        if all([blog_form.is_valid(), media_form.is_valid()]):
            media = media_form.save(commit=False)
            media.uploader = request.user
            media.save()
            blog_post = blog_form.save(commit=False)
            blog_post.author = request.user
            blog_post.photo = media
            blog_post.save()
            messages.add_message(request,
                                 messages.INFO,
                                 'Blog Post ajouté avec succès')
            return redirect('home')
        return render(request, self.template_name, self.context)


class PostEditView(LoginRequiredMixin, View):
    context = dict()
    template_name = 'blog/post_edit.html'
    edit_form_class = BlogForm
    delete_form_class = BlogDeleteForm

    def get(self, request, post_id):
        post = get_object_or_404(Blog, id=post_id)
        self.context['edit_form'] = self.edit_form_class(instance=post)
        self.context['delete_form'] = self.delete_form_class()
        return render(request, self.template_name, self.context)

    def post(self, request, post_id):
        post = get_object_or_404(Blog, id=post_id)

        if ('post_edit' in request.POST):
            self.context['edit_form'] = edit_form = self.edit_form_class(request.POST,
                                                          instance=post)
            if (edit_form.is_valid()):
                edit_form.save()
                return redirect('blog-post-view', post_id)

        if ('post_delete' in request.POST):
            self.context['delete_form'] = delete_form = self.delete_form_class(request.POST)
            if (delete_form.is_valid()):
                post.delete()
                return redirect('blog-post-list')

        return render(request, self.template_name, self.context)


def media_list_view(request):
    context = dict()
    context['medias'] = Photo.objects.all()
    return render(request, 'blog/media_list.html', context)


def media_view(request, media_id):
    context = dict()
    context['media'] = get_object_or_404(Photo, id=media_id)
    return render(request, 'blog/media_view.html', context)


def post_list_view(request):
    context = dict()
    context['posts'] = Blog.objects.all()
    return render(request, 'blog/post_list.html', context)


def post_view(request, post_id):
    context = dict()

    context['post'] = get_object_or_404(Blog, id=post_id)

    return render(request, 'blog/post_view.html', context)
